// ball_data.cc - deals with data required by balls
//
// Copyright (C) 2000 Trevor Spiteri
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#include <map>
#include <string>

#include "ball.hh"
#include "record.hh"

namespace {
    // holds required maps, local only to this file
    class maps {
    private:
        typedef std::map<const std::string, balls::new_ball_fn> mnn;

        mnn nbf_map;
    public:
        maps();
        ~maps();

        balls::new_ball_fn nbf(const std::string& name) const;
    };

    maps mapsi;

} // namespace

// return the function to create a new ball from the name
balls::new_ball_fn balls::name_to_nbf(const std::string& name)
{
    return mapsi.nbf(name);
}

maps::maps() {
    static bool done = false;
    if (!done) {
        done = true;
        nbf_map["none"] = 0;
        nbf_map["normal_ball"] = balls::normal_ball::new_ball;
        nbf_map["hard_ball"] = balls::hard_ball::new_ball;
        nbf_map["fast_ball"] = balls::fast_ball::new_ball;
        nbf_map["big_ball"] = balls::big_ball::new_ball;
        nbf_map["whiz_ball"] = balls::whiz_ball::new_ball;
    }
}

maps::~maps()
{
}

balls::new_ball_fn maps::nbf(const std::string& name) const
{
    mnn::const_iterator i = nbf_map.find(name);
    if (i == nbf_map.end())
        throw rec::not_found();
    return i->second;
}
