// game.hh - gives the game class
//
// Copyright (C) 2000 Trevor Spiteri
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#ifndef GAME_HH
#define GAME_HH

#include <list>

#include "ball.hh"
#include "graphics.hh"
#include "controller.hh"
#include "timer.hh"
#include "misc.hh"
#include "record.hh"

namespace Game {

    class cpanel {
    public:
        cpanel(graphics::tsurface* t, int x, int y, int w, int h,
               int vmin, int vmax, int smin, int smax,
               int hmin, int hmax, int amin, int amax,
               const rec::record& filenames);
        ~cpanel();
        
        void draw() const;

        int score;
        int depth;
        int time;

        int velocity, aminv, cminv, cmaxv, amaxv;
        int strength, amins, cmins, cmaxs, amaxs;
        int hardness, aminh, cminh, cmaxh, amaxh;
        int ammo, amina, cmina, cmaxa, amaxa;
    private:
        enum image_no {
            iscore = 10, idepth, itime,
            ivel, istr, ihard, iammo, itotal
        };
        graphics::image* images[itotal];

        enum dial_no {
            dvel, dstr, dhard, dammo, dtotal
        };
        graphics::dial* dials[dtotal];

        static const int draw_items = 14;
        int ys[draw_items + 1];

        void draw_num(int n, int x, int y) const;

        graphics::tsurface* ts;
        int X, Y;

        int unitx, unity;
    };

    using balls::ball; 

    class game {
    public:
        enum status {
            start, run, pause, done
        };

        game(int xr, int yr, const rec::record& options,
             graphics::tsurface* ts, rec::recrecord* b_d,
             control::controller* c, Timer::timer* t, misc::random& r);
        ~game();
   
        void play();
        void draw();

        int score() const         { return cp.score; }
        double depth_moved() const   { return Depth_moved; }
        int time_passed() const   { return Time_passed; }
        
        status stat() const       { return Stat; }
        int cycle_dur() const     { return Cycle_dur; }
        int timer_cycle_dur() const { return Timer_cycle_dur; }
   
        void set_cycle_dur(int cycle_dur) { Cycle_dur = cycle_dur; }
        void set_timer_cycle_dur(int dur) { Timer_cycle_dur = dur; }

    private:
        int xres, yres;
        int getx(int x) const { return x * xres / 1000; }
        int gety(int y) const { return y * yres / 1000; }

        int Time_passed;
        double Depth_moved;
        cpanel cp;
        int wicon_x, wicon_y;
        status Stat;
        int Cycle_dur, Timer_cycle_dur, max_time_wo_draw, time_since_last_draw;
        int time_wo_ships, time_left;

        misc::random& rnd;

        rec::recrecord* bd;

        graphics::tsurface* where_to_draw;
        misc::projector<> proj;
        misc::mapper<> mapp;

        control::controller* cont;
        Timer::timer* tmr;

        graphics::tunnel_drawable* tun;

        graphics::deep_image_map gallery;
        graphics::image_map wicons;

        ball* zero_depth_ball;
        std::list<ball*> ships;
        std::list<ball*> flyers;
        balls::creator* crea;
        balls::swallow_ball* swallower;

        balls::draw_ball_info db_info;

        void move_flyers();
        void sort_flyers();
        void check_flyers_depth_and_collisions();

        int intvel(double d) { return int(d * velconv); }
        int intdepth(double d) { return int(d * depthconv); }
        static const double velconv;
        static const double depthconv;
    };

} // namespace Game

#endif // GAME_HH
